! Copyright 2005-2007 ECMWF
! 
! Licensed under the GNU Lesser General Public License which
! incorporates the terms and conditions of version 3 of the GNU
! General Public License.
! See LICENSE and gpl-3.0.txt for details.
!
!
!  Description: how to create a new GRIB message from a template.
!               
!
!
!  Author: Anne Fouilloux
!
!
!
program template
  use grib_api
  implicit none  
  integer, parameter                            :: NbPressureLevel = 10
  integer                                       :: err
  integer                                       :: nx, ny, i
  integer                                       :: outfile
  integer                                       :: igrib
  integer                                       :: date, level
  real                                          :: missingValue
  integer                                       :: indicatorOfParameter
  double precision, dimension(:,:), allocatable :: field2D 
  integer, dimension(NbPressureLevel)           :: levels=(/1,2,10,50,70,100,200,500,850,1000/)

  ! Initialisation

  date = 20080104
  indicatorOfParameter = 130 ! temperature in table 128 used in the template

  !     a new grib message is loaded from an existing template
  !     templates are searched in a default template path (use grib_info
  !     to see where it is. The default template path can be changed by
  !     setting the environment variable GRIB_TEMPLATES_PATH
  call grib_new_from_template(igrib, "GRIB1")

  ! Open a new GRIB file to write new GRIB messages
  call grib_open_file(outfile, 'out.grib1','w')

  call grib_get(igrib,"numberOfPointsAlongAParallel", nx)
  
  call grib_get(igrib,"numberOfPointsAlongAMeridian",ny)

  call grib_get(igrib,'lev',level)

  allocate(field2D(nx,ny),stat=err)

  if (err .ne. 0) then
     print*, 'Failed to allocate ', nx*ny, ' values'
     STOP
  end if  
  call generate_field(field2D, missingValue)
 
! Same date and same parameters for all the levels
  call grib_set(igrib,"dataDate", date)
  call grib_set(igrib,"indicatorOfParameter", indicatorOfParameter)

  ! Write new GRIBs from the same message for different pressure levels 
  do i=1, size(levels)
     call grib_set(igrib,'lev',levels(i))

     ! compute the field for each pressure level     
     call generate_field(field2D, missingValue)

     ! set the value of missingValue
     call grib_set(igrib, 'missingValue',missingValue)

     call grib_set(igrib,"bitmapPresent",1)

     ! use pack to create 1D values
     call grib_set(igrib,'values',pack(field2D, mask=.true.))
 
  !     write the new message to a file
     call grib_write(igrib,outfile)
  end do

  call grib_release(igrib)

  call grib_close_file(outfile)
  deallocate(field2D)

contains
!======================================
subroutine generate_field(gfield2D, missingValue)
 double precision, dimension(:,:) :: gfield2D
 real, intent(out)                :: missingValue

 integer, parameter               :: borders = 5
 integer                          :: i
 integer, dimension(2)            :: shape_gfield2D

 shape_gfield2D = shape(gfield2D)

 call random_number(gfield2D)
! define the value of missingValue
 missingValue = 99999
! just keep the boundaries for gfield2D
  gfield2D(borders+1:shape_gfield2D(1)-borders,borders+1:shape_gfield2D(2)-borders) = missingValue

end subroutine generate_field
!======================================

end program template
